﻿using ComPDFKit.PDFDocument;
using ComPDFKit.PDFDocument.Action;
using ComPDFKitViewer.PdfViewer;
using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;

namespace compdfkit_tools.PDFControl
{
    /// <summary>
    /// PDFViewControl.xaml 的交互逻辑
    /// </summary>
    public partial class PDFViewControl : UserControl
    {
        public CPDFViewer PDFView { get;private set; }

        private double[] zoomLevelList = { 1f, 8f, 12f, 25, 33f, 50, 66f, 75, 100, 125, 150, 200, 300, 400, 600, 800, 1000 };

        public PDFViewControl()
        {
            InitializeComponent();
            PDFView = new CPDFViewer();
            Content = PDFView;
            PDFView.MouseWheelZoomHandler += PDFView_MouseWheelZoomHandler;
            PDFView.PDFActionHandler += PDFView_PDFActionHandler;
        }

        private void PDFView_PDFActionHandler(object sender,CPDFAction pdfAction)
        {
            if (pdfAction != null)
            {
                switch (pdfAction.ActionType)
                {
                    case C_ACTION_TYPE.ACTION_TYPE_NAMED:
                        {
                            CPDFNamedAction namedAction = pdfAction as CPDFNamedAction;
                            string namedStr = namedAction.GetName();
                            switch (namedStr)
                            {
                                case "FirstPage":
                                    {
                                        PDFView?.GoToPage(0);
                                        break;
                                    }
                                case "LastPage":
                                    {
                                        PDFView?.GoToPage(PDFView.Document.PageCount - 1);
                                        break;
                                    }
                                case "NextPage":
                                    if (PDFView != null)
                                    {
                                        int nextIndex = PDFView.CurrentIndex + 1;
                                        if (nextIndex < PDFView.Document.PageCount)
                                        {
                                            PDFView.GoToPage(nextIndex);
                                        }
                                    }
                                    break;
                                case "PrevPage":
                                    if (PDFView != null)
                                    {
                                        int prevIndex = PDFView.CurrentIndex - 1;
                                        if (prevIndex >= 0)
                                        {
                                            PDFView.GoToPage(prevIndex);
                                        }
                                    }
                                    break;
                                default:
                                    break;
                            }
                            break;
                        }

                    case C_ACTION_TYPE.ACTION_TYPE_GOTO:
                        if (PDFView != null)
                        {
                            CPDFGoToAction gotoAction = pdfAction as CPDFGoToAction;
                            CPDFDestination dest = gotoAction.GetDestination(PDFView.Document);
                            if (dest != null)
                                PDFView.GoToPage(dest.PageIndex, new Point(0, 0));
                        }
                        break;
                    case C_ACTION_TYPE.ACTION_TYPE_GOTOR:
                        if (PDFView != null)
                        {
                            CPDFGoToRAction gotorAction = pdfAction as CPDFGoToRAction;
                            CPDFDestination dest = gotorAction.GetDestination(PDFView.Document);
                            if (dest != null)
                            {
                                PDFView.GoToPage(dest.PageIndex, new Point(0, 0));
                            }
                        }
                        break;

                    case C_ACTION_TYPE.ACTION_TYPE_URI:
                        {
                            CPDFUriAction uriAction = pdfAction as CPDFUriAction;
                            string uri = uriAction.GetUri();
                            try
                            {
                                if (!string.IsNullOrEmpty(uri))
                                {
                                    Process.Start(uri);
                                }
                            }
                            catch (Exception ex)
                            {

                            }
                        }
                        break;

                    default:
                        break;
                }
            }

          
        }

        private void PDFView_MouseWheelZoomHandler(object sender, bool e)
        {
            double newZoom = CheckZoomLevel(PDFView.ZoomFactor + (e ? 0.01 : -0.01), e);
            PDFView.Zoom(newZoom);
        }

        private double CheckZoomLevel(double zoom, bool IsGrowth)
        {
            double standardZoom = 100;
            if (zoom <= 0.01)
            {
                return 0.01;
            }
            if (zoom >= 10)
            {
                return 10;
            }

            zoom *= 100;
            for (int i = 0; i < zoomLevelList.Length - 1; i++)
            {
                if (zoom > zoomLevelList[i] && zoom <= zoomLevelList[i + 1] && IsGrowth)
                {
                    standardZoom = zoomLevelList[i + 1];
                    break;
                }
                if (zoom >= zoomLevelList[i] && zoom < zoomLevelList[i + 1] && !IsGrowth)
                {
                    standardZoom = zoomLevelList[i];
                    break;
                }
            }
            return standardZoom / 100;
        }

    }
}
