﻿using ComPDFKit.PDFDocument;
using compdfkit_tools.Helper;
using compdfkit_tools.PDFControl;
using ComPDFKitViewer.AnnotEvent;
using ComPDFKitViewer;
using Microsoft.Win32;
using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.IO;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Controls.Primitives;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using System.Drawing;
using ComPDFKit.PDFPage;
using ComPDFKitViewer.PdfViewer;
using compdfkit_tools.Form;

namespace form_ctrl_demo
{
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {
        private PDFViewControl passwordViewer;
        private PDFViewControl pdfViewControl;
        private FromPropertyControl fromPropertyControl;

        bool LoadPDFFormTool = false;

        private double[] zoomLevelList = { 1f, 8f, 12f, 25, 33f, 50, 66f, 75, 100, 125, 150, 200, 300, 400, 600, 800, 1000 };
        public bool CanSave
        {
            get
            {
                if (pdfViewControl != null && pdfViewControl.PDFView != null)
                {
                    return pdfViewControl.PDFView.UndoManager.CanSave;
                }
                return false;
            }
        }
        public MainWindow()
        {
            InitializeComponent();
            Loaded += MainWindow_Loaded;
            DataContext = this;
        }

        private void MainWindow_Loaded(object sender, RoutedEventArgs e)
        {
            fromPropertyControl = new FromPropertyControl();
            LoadDefaultDocument();
            BindZoomLevel();
        }

        private void BindZoomLevel()
        {
            foreach (double zoomLevel in zoomLevelList)
            {
                ComboBoxItem zoomItem = new ComboBoxItem();
                zoomItem.Content = zoomLevel + "%";
                ZoomComboBox.Items.Add(zoomItem);
            }
        }

        private void LoadDocument()
        {
            pdfViewControl.PDFView?.Load();
            pdfViewControl.PDFView?.SetShowLink(true);
            PDFGrid.Child = pdfViewControl;
            pdfViewControl.PDFView.InfoChanged -= PdfViewer_InfoChanged;
            pdfViewControl.PDFView.InfoChanged += PdfViewer_InfoChanged;
            pdfViewControl.PDFView.AnnotCommandHandler -= PDFView_AnnotCommandHandler;
            pdfViewControl.PDFView.AnnotCommandHandler += PDFView_AnnotCommandHandler;
            PasswordUI.Closed -= PasswordUI_Closed;
            PasswordUI.Canceled -= PasswordUI_Canceled;
            PasswordUI.Confirmed -= PasswordUI_Confirmed;
            PasswordUI.Closed += PasswordUI_Closed;
            PasswordUI.Canceled += PasswordUI_Canceled;
            PasswordUI.Confirmed += PasswordUI_Confirmed;

            UIElement currentBotaTool = GetBotaTool();
            if (currentBotaTool is CPDFSearchControl)
            {
                ((CPDFSearchControl)currentBotaTool).InitWithPDFViewer(pdfViewControl.PDFView);
            }

            if (currentBotaTool is CPDFThumbnailControl)
            {
                ((CPDFThumbnailControl)currentBotaTool).InitWithPDFViewer(pdfViewControl.PDFView);
                ((CPDFThumbnailControl)currentBotaTool).ThumbLoaded = false;
                ((CPDFThumbnailControl)currentBotaTool).LoadThumb();
            }

            if (currentBotaTool is CPDFBookmarkControl)
            {
                ((CPDFBookmarkControl)currentBotaTool).InitWithPDFViewer(pdfViewControl.PDFView);
                ((CPDFBookmarkControl)currentBotaTool).LoadBookmark();
            }

            ViewSettingBtn.IsChecked = false;
            PropertyContainer.Child = null;
            PropertyContainer.Visibility = Visibility.Collapsed;
            ZoomTextBox.Text = string.Format("{0}", (int)(pdfViewControl.PDFView.ZoomFactor * 100)) + "%";

            InitialPDFViewControl(pdfViewControl);
        }

        private void PDFView_AnnotCommandHandler(object sender, AnnotCommandArgs e)
        {
            if (e != null && e.CommandType == CommandType.Context)
            {
                if (e.PressOnSelectedText)
                {
                    e.Handle = true;
                    e.PopupMenu = new ContextMenu();
                    e.PopupMenu.Items.Add(new MenuItem() { Header = "Copy", Command = ApplicationCommands.Copy, CommandTarget = (UIElement)sender });
                }
                else if (e.CommandTarget == TargetType.ImageSelection)
                {
                    if (pdfViewControl != null && pdfViewControl.PDFView != null && pdfViewControl.PDFView.GetSelectImageCount() > 0)
                    {
                        e.Handle = true;
                        e.PopupMenu = new ContextMenu();

                        MenuItem imageCopyMenu = new MenuItem();
                        imageCopyMenu = new MenuItem();
                        imageCopyMenu.Header = "Copy Images";
                        WeakEventManager<MenuItem, RoutedEventArgs>.AddHandler(imageCopyMenu, "Click", CopyImage_Click);
                        imageCopyMenu.CommandParameter = e;
                        e.PopupMenu.Items.Add(imageCopyMenu);

                        MenuItem imageExtraMenu = new MenuItem();
                        imageExtraMenu = new MenuItem();
                        imageExtraMenu.Header = "Extract Images";
                        WeakEventManager<MenuItem, RoutedEventArgs>.AddHandler(imageExtraMenu, "Click", ExtraImage_Click);
                        imageExtraMenu.CommandParameter = e;
                        e.PopupMenu.Items.Add(imageExtraMenu);
                    }
                }
                else if (e.CommandTarget == TargetType.WidgetView)
                {
                    e.Handle = true;
                    e.PopupMenu = new ContextMenu();
                    e.PopupMenu.Items.Add(new MenuItem() { Header = "Copy", Command = ApplicationCommands.Copy, CommandTarget = (UIElement)sender });
                    e.PopupMenu.Items.Add(new MenuItem() { Header = "Cut", Command = ApplicationCommands.Cut, CommandTarget = (UIElement)sender });
                    e.PopupMenu.Items.Add(new MenuItem() { Header = "Delete", Command = ApplicationCommands.Delete, CommandTarget = (UIElement)sender });
                }
                else
                {
                    e.Handle = true;
                    e.PopupMenu = new ContextMenu();

                    e.PopupMenu.Items.Add(new MenuItem() { Header = "Paste", Command = ApplicationCommands.Paste, CommandTarget = (UIElement)sender });
                    e.PopupMenu.Items.Add(new Separator());

                    MenuItem fitWidthMenu = new MenuItem();
                    fitWidthMenu.Header = "Fit Width";
                    fitWidthMenu.Click += (o, p) =>
                    {
                        if (pdfViewControl != null)
                        {
                            pdfViewControl.PDFView?.ChangeFitMode(FitMode.FitWidth);
                        }
                    };

                    e.PopupMenu.Items.Add(fitWidthMenu);

                    MenuItem fitSizeMenu = new MenuItem();
                    fitSizeMenu.Header = "Actual Size";
                    fitSizeMenu.Click += (o, p) =>
                    {
                        if (pdfViewControl != null)
                        {
                            pdfViewControl.PDFView?.ChangeFitMode(FitMode.FitSize);
                        }
                    };

                    e.PopupMenu.Items.Add(fitSizeMenu);

                    MenuItem zoomInMenu = new MenuItem();
                    zoomInMenu.Header = "Zoom In";
                    zoomInMenu.Click += (o, p) =>
                    {
                        if (pdfViewControl != null)
                        {
                            double newZoom = CheckZoomLevel(pdfViewControl.PDFView.ZoomFactor + 0.01, true);
                            pdfViewControl.PDFView?.Zoom(newZoom);
                        }
                    };

                    e.PopupMenu.Items.Add(zoomInMenu);

                    MenuItem zoomOutMenu = new MenuItem();
                    zoomOutMenu.Header = "Zoom Out";
                    zoomOutMenu.Click += (o, p) =>
                    {
                        if (pdfViewControl != null)
                        {
                            double newZoom = CheckZoomLevel(pdfViewControl.PDFView.ZoomFactor - 0.01, false);
                            pdfViewControl.PDFView?.Zoom(newZoom);
                        }
                    };

                    e.PopupMenu.Items.Add(zoomOutMenu);
                    e.PopupMenu.Items.Add(new Separator());

                    MenuItem singleView = new MenuItem();
                    singleView.Header = "Single Page";
                    singleView.Click += (o, p) =>
                    {
                        if (pdfViewControl != null)
                        {
                            pdfViewControl.PDFView?.ChangeViewMode(ViewMode.Single);
                        }
                    };

                    e.PopupMenu.Items.Add(singleView);

                    MenuItem singleContinuousView = new MenuItem();
                    singleContinuousView.Header = "Single Page Continuous";
                    singleContinuousView.Click += (o, p) =>
                    {
                        if (pdfViewControl != null)
                        {
                            pdfViewControl.PDFView?.ChangeViewMode(ViewMode.SingleContinuous);
                        }
                    };

                    e.PopupMenu.Items.Add(singleContinuousView);

                    MenuItem doubleView = new MenuItem();
                    doubleView.Header = "Two Pages";
                    doubleView.Click += (o, p) =>
                    {
                        if (pdfViewControl != null)
                        {
                            pdfViewControl.PDFView?.ChangeViewMode(ViewMode.Double);
                        }
                    };

                    e.PopupMenu.Items.Add(doubleView);

                    MenuItem doubleContinuousView = new MenuItem();
                    doubleContinuousView.Header = "Two Pages Continuous";
                    doubleContinuousView.Click += (o, p) =>
                    {
                        if (pdfViewControl != null)
                        {
                            pdfViewControl.PDFView?.ChangeViewMode(ViewMode.DoubleContinuous);
                        }
                    };
                    e.PopupMenu.Items.Add(doubleContinuousView);
                }
            }
            else
            {
                e.DoCommand();
            }
        }

        private void PasswordUI_Confirmed(object sender, string e)
        {
            if (passwordViewer != null && passwordViewer.PDFView != null && passwordViewer.PDFView.Document != null)
            {
                passwordViewer.PDFView.Document.UnlockWithPassword(e);
                if (passwordViewer.PDFView.Document.IsLocked == false)
                {
                    PasswordUI.SetShowError("", Visibility.Collapsed);
                    PasswordUI.ClearPassword();
                    PasswordUI.Visibility = Visibility.Collapsed;
                    PopupBorder.Visibility = Visibility.Collapsed;
                    pdfViewControl = passwordViewer;
                    LoadDocument();
                }
                else
                {
                    PasswordUI.SetShowError("error", Visibility.Visible);
                }
            }
        }

        private void PasswordUI_Canceled(object sender, EventArgs e)
        {
            PopupBorder.Visibility = Visibility.Collapsed;
            PasswordUI.Visibility = Visibility.Collapsed;
        }

        private void PasswordUI_Closed(object sender, EventArgs e)
        {
            PopupBorder.Visibility = Visibility.Collapsed;
            PasswordUI.Visibility = Visibility.Collapsed;
        }

        private void PdfViewer_InfoChanged(object sender, KeyValuePair<string, object> e)
        {
            if (e.Key == "PageNum")
            {
                PageRangeText.Text = string.Format("{0}/{1}", e.Value, pdfViewControl.PDFView.Document.PageCount);
            }

            if (e.Key == "Zoom")
            {
                ZoomTextBox.Text = string.Format("{0}", (int)((double)e.Value * 100)) + "%";
            }
        }


        private double CheckZoomLevel(double zoom, bool IsGrowth)
        {
            double standardZoom = 100;
            if (zoom <= 0.01)
            {
                return 0.01;
            }
            if (zoom >= 10)
            {
                return 10;
            }

            zoom *= 100;
            for (int i = 0; i < zoomLevelList.Length - 1; i++)
            {
                if (zoom > zoomLevelList[i] && zoom <= zoomLevelList[i + 1] && IsGrowth)
                {
                    standardZoom = zoomLevelList[i + 1];
                    break;
                }
                if (zoom >= zoomLevelList[i] && zoom < zoomLevelList[i + 1] && !IsGrowth)
                {
                    standardZoom = zoomLevelList[i];
                    break;
                }
            }
            return standardZoom / 100;
        }
        private void LoadDefaultDocument()
        {
            string defaultFilePath = "developer_guide_windows.pdf";
            pdfViewControl = new PDFViewControl();
            pdfViewControl.PDFView.InitDocument(defaultFilePath);
            LoadDocument();
        }

        /// <summary>
        /// 搜索工具点击处理
        /// </summary>
        private void SearchToolButton_Click(object sender, RoutedEventArgs e)
        {
            UIElement botaTool = GetBotaTool();
            if (botaTool == null || !(botaTool is CPDFSearchControl))
            {
                CPDFSearchControl searchControl = new CPDFSearchControl();
                if (pdfViewControl != null && pdfViewControl.PDFView != null && pdfViewControl.PDFView.Document != null)
                {
                    searchControl.InitWithPDFViewer(pdfViewControl.PDFView);
                }
                SetBotaTool(searchControl);
            }
            ExpandBotaTool(SearchToolButton.IsChecked == true);
            ClearToolState(SearchToolButton);
        }

        /// <summary>
        /// 缩略图列表点击处理
        /// </summary>
        private void ThumbToolButton_Click(object sender, RoutedEventArgs e)
        {
            UIElement botaTool = GetBotaTool();
            if (botaTool == null || !(botaTool is CPDFThumbnailControl))
            {
                CPDFThumbnailControl thumbControl = new CPDFThumbnailControl();

                if (pdfViewControl != null && pdfViewControl.PDFView != null && pdfViewControl.PDFView.Document != null)
                {
                    thumbControl.InitWithPDFViewer(pdfViewControl.PDFView);
                    thumbControl.LoadThumb();
                }
                SetBotaTool(thumbControl);
            }
            ExpandBotaTool(ThumbToolButton.IsChecked == true);
            ClearToolState(ThumbToolButton);
        }

        /// <summary>
        /// 书签列表点击处理
        /// </summary>
        private void BookmarkToolButtonn_Click(object sender, RoutedEventArgs e)
        {
            UIElement botaTool = GetBotaTool();
            if (botaTool == null || !(botaTool is CPDFBookmarkControl))
            {
                CPDFBookmarkControl bookmarkControl = new CPDFBookmarkControl();

                if (pdfViewControl != null && pdfViewControl.PDFView != null && pdfViewControl.PDFView.Document != null)
                {
                    bookmarkControl.InitWithPDFViewer(pdfViewControl.PDFView);
                    bookmarkControl.LoadBookmark();
                }
                SetBotaTool(bookmarkControl);
            }
            ExpandBotaTool(BookmarkToolButton.IsChecked == true);
            ClearToolState(BookmarkToolButton);
        }

        /// <summary>
        /// 大纲列表处理
        /// </summary>
        private void OutlineToolButton_Click(object sender, RoutedEventArgs e)
        {
            UIElement botaTool = GetBotaTool();
            if (botaTool == null || !(botaTool is CPDFOutlineControl))
            {
                CPDFOutlineControl outlineControl = new CPDFOutlineControl();

                if (pdfViewControl != null && pdfViewControl.PDFView != null && pdfViewControl.PDFView.Document != null)
                {
                    outlineControl.InitWithPDFViewer(pdfViewControl.PDFView);
                }
                SetBotaTool(outlineControl);
            }
            ExpandBotaTool(OutlineToolButton.IsChecked == true);
            ClearToolState(OutlineToolButton);

        }
        /// <summary>
        /// 设置Bota工具内容
        /// </summary>
        /// <param name="newChild"></param>
        private void SetBotaTool(UIElement newChild)
        {
            BotaToolContainer.Child = newChild;
        }

        /// <summary>
        /// 获取Bota工具
        /// </summary>
        /// <returns></returns>
        private UIElement GetBotaTool()
        {
            return BotaToolContainer.Child;
        }

        /// <summary>
        /// 展开Bota工具
        /// </summary>
        /// <param name="isExpand"></param>
        private void ExpandBotaTool(bool isExpand)
        {
            BotaToolContainer.Visibility = isExpand ? Visibility.Visible : Visibility.Collapsed;
            Splitter.Visibility = isExpand ? Visibility.Visible : Visibility.Collapsed;
        }

        /// <summary>
        /// 清除左边工具栏状态
        /// </summary>
        private void ClearToolState(UIElement ignoreTool)
        {
            foreach (UIElement child in BotaSideTool.Children)
            {
                if (child != ignoreTool && child is ToggleButton buttonTool)
                {
                    buttonTool.IsChecked = false;
                }
            }
        }

        private void ToolExpand_Click(object sender, RoutedEventArgs e)
        {
            ToggleButton expandBtn = sender as ToggleButton;
            if (expandBtn != null)
            {
                bool isExpand = expandBtn.IsChecked == true;
                ExpandLeftPanel(isExpand);
                if (isExpand)
                {
                    ThumbToolButton.IsChecked = isExpand;
                    ThumbToolButton_Click(ThumbToolButton, new RoutedEventArgs());
                }
            }
        }

        private void ExpandSearchBtn_Click(object sender, RoutedEventArgs e)
        {
            ExpandLeftPanel(true);
            SearchToolButton.IsChecked = true;
            ToolExpandBtn.IsChecked = true;
            SearchToolButton_Click(SearchToolButton, new RoutedEventArgs());
        }

        private void ExpandLeftPanel(bool isExpand)
        {
            ExpandToolContainer.Visibility = isExpand ? Visibility.Visible : Visibility.Collapsed;
            Splitter.Visibility = isExpand ? Visibility.Visible : Visibility.Collapsed;
            if (isExpand)
            {
                BodyGrid.ColumnDefinitions[0].Width = new GridLength(260);
                BodyGrid.ColumnDefinitions[1].Width = new GridLength(15);
            }
            else
            {
                BodyGrid.ColumnDefinitions[0].Width = new GridLength(0);
                BodyGrid.ColumnDefinitions[1].Width = new GridLength(0);
            }
        }

        private void ViewSettingBtn_Click(object sender, RoutedEventArgs e)
        {
            ToggleButton toggleButton = sender as ToggleButton;
            if (toggleButton != null)
            {
                if (toggleButton.IsChecked == true)
                {
                    CPDFDisplaySettingsControl displayPanel = new CPDFDisplaySettingsControl();
                    displayPanel.InitWithPDFViewer(pdfViewControl.PDFView);
                    PropertyContainer.Child = displayPanel;
                    PropertyContainer.Visibility = Visibility.Visible;
                    if ((bool)FormBarBtn.IsChecked)
                    {
                        FormBarBtn.IsChecked = false;
                    }
                }
                else
                {
                    PropertyContainer.Child = null;
                    PropertyContainer.Visibility = Visibility.Collapsed;
                }
            }
        }

        private void FormBarBtn_Click(object sender, RoutedEventArgs e)
        {
            ToggleButton toggleButton = sender as ToggleButton;
            if (toggleButton != null)
            {
                if (toggleButton.IsChecked == true)
                {
                    //if (pdfAnnotationControl != null)
                    //{
                    ExpandRightPropertyPanel(fromPropertyControl, Visibility.Visible);
                    if ((bool)ViewSettingBtn.IsChecked)
                    {
                        ViewSettingBtn.IsChecked = false;
                    }
                    //}
                }
                else
                {
                    ExpandRightPropertyPanel(null, Visibility.Collapsed);
                }
            }
        }

        private void ZoomComboBox_SelectionChanged(object sender, SelectionChangedEventArgs e)
        {
            ComboBoxItem selectItem = ZoomComboBox.SelectedItem as ComboBoxItem;
            if (selectItem != null && selectItem.Content != null && pdfViewControl != null)
            {
                if (double.TryParse(selectItem.Content.ToString().TrimEnd('%'), out double zoomLevel))
                {
                    pdfViewControl.PDFView.Zoom(zoomLevel / 100D);
                    ZoomTextBox.Text = string.Format("{0}", (int)(pdfViewControl.PDFView.ZoomFactor * 100)) + "%";
                }
            }
        }

        private void ZoomInBtn_Click(object sender, RoutedEventArgs e)
        {
            if (pdfViewControl != null)
            {
                double newZoom = CheckZoomLevel(pdfViewControl.PDFView.ZoomFactor + 0.01, true);
                pdfViewControl.PDFView.Zoom(newZoom);
                ZoomTextBox.Text = string.Format("{0}", (int)(newZoom * 100)) + "%";
            }
        }

        private void ZoomOutBtn_Click(object sender, RoutedEventArgs e)
        {
            if (pdfViewControl != null)
            {
                double newZoom = CheckZoomLevel(pdfViewControl.PDFView.ZoomFactor - 0.01, false);
                pdfViewControl.PDFView.Zoom(newZoom);
                ZoomTextBox.Text = string.Format("{0}", (int)(newZoom * 100)) + "%";
            }
        }

        private void NextPageBorder_MouseLeftButtonDown(object sender, MouseButtonEventArgs e)
        {
            pdfViewControl.PDFView?.GoToPage(pdfViewControl.PDFView.CurrentIndex + 1);
        }

        private void PrevPageBorder_MouseLeftButtonDown(object sender, MouseButtonEventArgs e)
        {
            pdfViewControl.PDFView?.GoToPage(pdfViewControl.PDFView.CurrentIndex - 1);
        }

        private void PageInfoBtn_Click(object sender, RoutedEventArgs e)
        {
            PasswordUI.Visibility = Visibility.Collapsed;
            FileInfoUI.Visibility = Visibility.Visible;
            FileInfoControl.InitWithPDFViewer(pdfViewControl.PDFView);
            PopupBorder.Visibility = Visibility.Visible;
        }
        public void ExpandRightPropertyPanel(UIElement properytPanel, Visibility visible)
        {
            PropertyContainer.Width = 260;
            PropertyContainer.Child = properytPanel;
            PropertyContainer.Visibility = visible;
        }

        private void FileInfoCloseBtn_Click(object sender, RoutedEventArgs e)
        {
            PopupBorder.Visibility = Visibility.Collapsed;
        }

        private void OpenFile_Click(object sender, RoutedEventArgs e)
        {
            try
            {
                string filePath = CommonHelper.GetFilePathOrEmpty();
                if (!string.IsNullOrEmpty(filePath) && pdfViewControl != null)
                {
                    if (pdfViewControl.PDFView != null && pdfViewControl.PDFView.Document != null)
                    {
                        string oldFilePath = pdfViewControl.PDFView.Document.FilePath;
                        if (oldFilePath.ToLower() == filePath.ToLower())
                        {
                            return;
                        }
                    }

                    passwordViewer = new PDFViewControl();
                    passwordViewer.PDFView.InitDocument(filePath);
                    if (passwordViewer.PDFView.Document == null)
                    {
                        MessageBox.Show("Open File Failed");
                        return;
                    }

                    if (passwordViewer.PDFView.Document.IsLocked)
                    {
                        PasswordUI.SetShowText(System.IO.Path.GetFileName(filePath) + " password encrypted.");
                        PasswordUI.ClearPassword();
                        PopupBorder.Visibility = Visibility.Visible;
                        PasswordUI.Visibility = Visibility.Visible;
                    }
                    else
                    {
                        pdfViewControl = passwordViewer;
                        LoadDocument();
                    }
                }
            }
            catch (Exception ex)
            {

            }

        }

        private void SaveFileBtn_Click(object sender, RoutedEventArgs e)
        {
            if (pdfViewControl != null && pdfViewControl.PDFView != null && pdfViewControl.PDFView.Document != null)
            {
                try
                {
                    CPDFDocument pdfDoc = pdfViewControl.PDFView.Document;
                    if (pdfDoc.WriteToLoadedPath())
                    {
                        return;
                    }

                    SaveFileDialog saveDialog = new SaveFileDialog();
                    saveDialog.Filter = "(*.pdf)|*.pdf";
                    saveDialog.DefaultExt = ".pdf";
                    saveDialog.OverwritePrompt = true;

                    if (saveDialog.ShowDialog() == true)
                    {
                        pdfDoc.WriteToFilePath(saveDialog.FileName);
                    }
                }
                catch (Exception ex)
                {

                }
            }
        }

        private void CopyImage_Click(object sender, RoutedEventArgs e)
        {
            try
            {
                Dictionary<int, List<Bitmap>> imageDict = pdfViewControl.PDFView?.GetSelectedImages();

                if (imageDict != null && imageDict.Count > 0)
                {
                    foreach (int pageIndex in imageDict.Keys)
                    {
                        List<Bitmap> imageList = imageDict[pageIndex];
                        foreach (Bitmap image in imageList)
                        {
                            MemoryStream ms = new MemoryStream();
                            image.Save(ms, System.Drawing.Imaging.ImageFormat.Png);
                            BitmapImage imageData = new BitmapImage();
                            imageData.BeginInit();
                            imageData.StreamSource = ms;
                            imageData.CacheOption = BitmapCacheOption.OnLoad;
                            imageData.EndInit();
                            imageData.Freeze();
                            Clipboard.SetImage(imageData);
                            break;
                        }
                    }
                }
            }
            catch (Exception ex)
            {

            }
        }

        private void ExtraImage_Click(object sender, RoutedEventArgs e)
        {
            System.Windows.Forms.FolderBrowserDialog folderDialog = new System.Windows.Forms.FolderBrowserDialog();
            if (folderDialog.ShowDialog() == System.Windows.Forms.DialogResult.OK)
            {
                string choosePath = folderDialog.SelectedPath;
                string openPath = choosePath;
                try
                {
                    Dictionary<int, List<Bitmap>> imageDict = pdfViewControl.PDFView?.GetSelectedImages();

                    if (imageDict != null && imageDict.Count > 0)
                    {
                        foreach (int pageIndex in imageDict.Keys)
                        {
                            List<Bitmap> imageList = imageDict[pageIndex];
                            foreach (Bitmap image in imageList)
                            {
                                string savePath = System.IO.Path.Combine(choosePath, Guid.NewGuid() + ".jpg");
                                image.Save(savePath, System.Drawing.Imaging.ImageFormat.Jpeg);
                                openPath = savePath;
                            }
                        }
                    }
                    Process.Start("explorer", "/select,\"" + openPath + "\"");
                }
                catch (Exception ex)
                {

                }

            }
        }

        private void ComboBox_SelectionChanged(object sender, SelectionChangedEventArgs e)
        {
            SetFromMode();
        }
        private void SetFromMode()
        {
            if (LoadPDFFormTool)
            {
                PDFFormTool.ClearAllToolState();
            }
            if (pdfViewControl != null && ViewComboBox != null)
            {
                if (ViewComboBox.SelectedIndex == 0)
                {
                    PDFFormTool.Visibility = Visibility.Collapsed;
                    pdfViewControl.PDFView?.SetShowLink(true);
                    pdfViewControl.PDFView?.SetPDFEditType(CPDFEditType.None);
                    pdfViewControl.PDFView?.SetMouseMode(MouseModes.PanTool);
                    pdfViewControl.PDFView?.ReloadDocument();
                    if (PropertyContainer != null)
                    {
                        fromPropertyControl.CleanProperty();
                        PropertyContainer.Width = 0;
                        PropertyContainer.Child = null;
                        ViewSettingBtn.IsChecked = false;
                        FormBarBtn.IsChecked = false;
                    }
                    return;
                }

                if (ViewComboBox.SelectedIndex == 1)
                {
                    PDFFormTool.Visibility = Visibility.Visible;
                    pdfViewControl.PDFView?.SetShowLink(false);
                    pdfViewControl.PDFView?.SetMouseMode(MouseModes.FormEditTool);
                    pdfViewControl.PDFView?.ReloadDocument();
                }
            }
        }

        public void InitialPDFViewControl(PDFViewControl newPDFViewer)
        {
            PDFFormTool.SetPDFViewer(newPDFViewer.PDFView, fromPropertyControl);
            fromPropertyControl.SetPDFViewer(newPDFViewer.PDFView);
            PDFFormTool.ClearAllToolState();
            newPDFViewer.PDFView.AnnotEditHandler += PDFView_AnnotEditHandler;
            newPDFViewer.PDFView.AnnotActiveHandler += PDFView_AnnotActiveHandler;
            newPDFViewer.PDFView.WidgetClickHandler += PDFView_WidgetClickHandler; ;
        }

        private void PDFView_WidgetClickHandler(object sender, WidgetArgs e)
        {
            if (e is WidgetSignArgs)
            {
                var x = 1;
            }
        }

        private void PDFView_AnnotActiveHandler(object sender, AnnotAttribEvent e)
        {
            if (e == null || e.IsAnnotCreateReset)
            { }
            else
            {
                switch (e.GetAnnotTypes())
                {
                    case AnnotArgsType.WidgetViewForm:
                        WidgetArgs formArgs = e.GetAnnotHandlerEventArgs(AnnotArgsType.WidgetViewForm).First() as WidgetArgs;
                        switch (formArgs.WidgeType)
                        {
                            case ComPDFKit.PDFAnnotation.Form.C_WIDGET_TYPE.WIDGET_PUSHBUTTON:
                            case ComPDFKit.PDFAnnotation.Form.C_WIDGET_TYPE.WIDGET_COMBOBOX:
                            case ComPDFKit.PDFAnnotation.Form.C_WIDGET_TYPE.WIDGET_LISTBOX:
                                ExpandRightPropertyPanel(fromPropertyControl, Visibility.Visible);
                                if ((bool)ViewSettingBtn.IsChecked)
                                {
                                    ViewSettingBtn.IsChecked = false;
                                }
                                FormBarBtn.IsChecked = true;
                                break;
                            default:
                                break;
                        }
                        fromPropertyControl.SetPropertyForType(formArgs, e);
                        break;
                }
            }
        }

        private void PDFView_AnnotEditHandler(object sender, List<AnnotEditEvent> e)
        {
            if (e != null && e.Count > 0)
            {
                AnnotEditEvent editEvent = e[e.Count - 1];
                if (editEvent.EditAction == ActionType.Add)
                {
                    pdfViewControl.PDFView.SelectAnnotation(editEvent.PageIndex, editEvent.AnnotIndex);
                }
            }
        }

        private void PDFFormTool_Loaded(object sender, RoutedEventArgs e)
        {
            LoadPDFFormTool = true;

        }

        private void PDFFormTool_Unloaded(object sender, RoutedEventArgs e)
        {
            LoadPDFFormTool = false;

        }
    }
}
